<?php
// $Id: Taxonews.php,v 1.1.2.4 2011/01/13 09:52:47 fgm Exp $
/**
 * @file
 * A module to display blocks selected by terms in a taxonomy vocabulary,
 * as long as they are newer than a set limit
 *
 * The module defines one block for each term in the vocabulary.
 *
 * Settings allows the administrator to choose
 *   - the vocabulary to be used to select nodes
 *   - the number of days nodes are considered for display
 *
 * Blocks without matching nodes do not appear.
 *
 * This file contains the class actually implementing the module
 *
 *
 * @copyright  2005-2011 Ouest Systèmes Informatiques SARL (OSInet)
 *
 * @author Frédéric G. MARAND
 *
 * @link       http://drupal.org/project/taxonews
 *
 * @since      Drupal 4.6
 *
 * @license Licensed under the CeCILL version 2 and GPL version 2 or later
 *
 * License note: G2 is distributed by OSInet to its customers under the
 * CeCILL 2.0 license. OSInet support services only apply to the module
 * when distributed by OSInet, not by any third-party further down the
 * distribution chain.
 *
 * If you obtained G2 from drupal.org, that site received it under the
 * GPL v2 or later license and can therefore distribute it under the
 * GPL v2 or later, and so can you and just anyone down the chain as
 * long as the GPL v2 or later terms are abided by, the module
 * distributor in that case being the drupal.org organization or the
 * downstream distributor, not OSInet.
 */

/**
 * This is the main class for taxonews module.
 *
 * - It contains the whole module logic
 * - It does not contain the default theme
 */
class Taxonews
  {
  const VERSION             = '$Id: Taxonews.php,v 1.1.2.4 2011/01/13 09:52:47 fgm Exp $ 2007-06-12';
  const PATH_SETTINGS       = 'admin/build/settings/taxonews';

  /**
   * Names of persistent variables
   */
  const VAR_LIFETIME        = 'taxonews_lifetime';
  const VAR_MAX_ROWS        = 'taxonews_max_rows';
  const VAR_SHOW_NAME       = 'taxonews_show_name';
  const VAR_SHOW_ARCHIVE    = 'taxonews_show_archive';
  const VAR_SHOW_EMPTY      = 'taxonews_show_empty';
  const VAR_EMPTY_MESSAGES  = 'taxonews_empty_messages';
  const VAR_FEED            = 'taxonews_feed';
  const VAR_PONDERATED      = 'taxonews_ponderated';
  const VAR_VOCABULARY      = 'taxonews_vid';

  /**
   * Default values for persistent variables
   */
  const DEF_LIFETIME        = 30;
  const DEF_MAX_ROWS        = 5;

  /**
   * Implement the former hook_settings
   *
   * @return array settings form
   */
  function adminSettings ()
    {
    $sq = 'SELECT v.vid, v.name, v.description '
       . 'FROM {vocabulary} v '
       . 'ORDER BY 2, 1 ';
    $sq = db_rewrite_sql($sq, 'v', 'vid');
    $q = db_query($sq);
    $vocabularies = array();
    while ($o = db_fetch_object($q))
      {
      $vocabularies["$o->vid"] = "$o->name - $o->description" ;
      }

    $form[self::VAR_VOCABULARY] = array
      (
      '#type'             => 'select',
      '#title'            => t('Vocabularies'),
      '#default_value'    => variable_get(self::VAR_VOCABULARY, 0),
      '#options'          => $vocabularies,
      '#description'      => t('Vocabularies to be used for the generation of blocks. A block will be defined for each term in each vocabulary selected here.'),
      '#required'         => TRUE,
      '#multiple'         => TRUE,
      '#size'             => count($vocabularies),
      );

    $form[self::VAR_PONDERATED] = array
      (
      '#type'             => 'select',
      '#title'            => t('Ponderate sorting by'),
      '#options'          => array
        (
        0 => t('Creation date'),
        ),
      );
    if (module_exists('statistics'))
      {
      $form[self::VAR_PONDERATED]['#options'] += array
        (
        1 => t('Total views'),
        2 => t('Daily views'),
        );
      $form[self::VAR_PONDERATED] += array
        (
        '#default_value'    => variable_get(self::VAR_PONDERATED, 0),
        '#description'      => t('Sort displayed news by descending creation date, possibly ponderated by total number of views and daily views. This ponderation only affects sorting, not news selection, which are always selected by creation date.'),
        );
      }
    else
      {
      $form[self::VAR_PONDERATED] += array
        (
        '#default_value'    => 0,
        '#description'      => t('Sort displayed news by descending creation date. Additional sortings will be available if you !enable. It is currently disabled.',
          array('!enable' => l(t('enable statistics.module'), 'admin/build/modules'))),
        );
      }

    $form[self::VAR_LIFETIME] = array
      (
      '#type'             => 'textfield',
      '#title'            => t('Lifetime of news before expiration'),
      '#default_value'    => variable_get(self::VAR_LIFETIME, self::DEF_LIFETIME),
      '#size'             => 3,
      '#max_length'       => 3,
      '#description'      => t('The number of days a news item remains displayed in blocks.'),
      '#required'         => TRUE
      )  ;
    $form[self::VAR_MAX_ROWS] = array
      (
      '#type'             => 'textfield',
      '#title'            => t('Maximum number of rows per block'),
      '#default_value'    => variable_get(self::VAR_MAX_ROWS, self::DEF_MAX_ROWS),
      '#size'             => 3,
      '#max_length'       => 3,
      '#description'      => t('The maximum number of rows that the builtin theme will display in a block.'),
      '#required'         => TRUE
      )  ;
    $form[self::VAR_FEED] = array
      (
      '#type'            => 'checkbox',
      '#title'           => t('Display RSS feed icon on taxonews blocks'),
      '#default_value'   => variable_get(self::VAR_FEED, TRUE),
      '#description'     => t('This setting allows taxonews to display a RSS feed icon on each non empty block. Default value is enabled.'),
      );
    $form[self::VAR_SHOW_NAME] = array
      (
      '#type'            => 'checkbox',
      '#title'           => t('Prepend taxonews module name in block list'),
      '#default_value'   => variable_get(self::VAR_SHOW_NAME, TRUE),
      '#description'     => t('This allows the modules to appear grouped on the block list at Administer/Blocks, to avoid cluttering.'),
      );
    $form[self::VAR_SHOW_ARCHIVE] = array
      (
      '#type'            => 'checkbox',
      '#title'           => t('Include "Archive" link to older articles in block'),
      '#default_value'   => variable_get(self::VAR_SHOW_ARCHIVE, TRUE),
      '#description'     => t('If articles matching the term exist, but cannot be placed in the block, for instance if they are too old, add an "Archive" link at the end of the block. The link will not be shown if no matching article exists.'),
      );
    $form[self::VAR_SHOW_EMPTY] = array
      (
      '#type'            => 'checkbox',
      '#title'           => t('Include blocks with currently no matching nodes in the blocks list'),
      '#default_value'   => variable_get(self::VAR_SHOW_EMPTY, TRUE),
      '#description'     => t('It is strongly advised to keep this setting to True. If set to False, the block numbers ($delta) will shift in case new content becomes categorized under a yet-empty term.')
      );
    $form['advanced'] = array
      (
      '#type'            => 'fieldset',
      '#collapsible'     => TRUE,
      '#collapsed'       => TRUE,
      '#title'           => t('Advanced settings'),
      );
    $form['advanced'][self::VERSION] = array
      (
      '#value'           => t('<p>This site is running taxonews.module version: @version.</p>',
        array('@version' => self::VERSION))
      );

    $ret = system_settings_form($form);
    return $ret;
    }

  /**
   * Is there any published data matching that term ?
   *
   * @param int $tid
   * @param array $nids Array of displayed nodes, to ignore
   * @return boolean
   *
   * The function expects the nids of the already displayed nodes
   * to be the keys of the $items array, so it can ignore them
   *
   * The {node} alias is set to 'm' instead of 'n' to avoid falling into
   * issue #284392, which is fixed in Drupal 6.16 but not in 5.x as of 5.22
   */
  static function archiveExists($tid, $items = array())
    {
    $sq = 'SELECT COUNT(tn.nid) '
        . 'FROM {node} m '
        . 'INNER JOIN {term_node} tn ON tn.nid = m.nid '
        . 'WHERE tn.tid = %d AND m.status = 1 ';

    if (!isset($items))
      {
      $items = array();
      }
    $ar_nids = array_keys($items);
    $s_nids = implode(',', $ar_nids);
    if (sizeof($ar_nids))
      {
      $sq .= 'AND tn.nid NOT IN (%s) ';
      $sq = db_rewrite_sql($sq, 'm'); //
      $q = db_query_range($sq, $tid, $s_nids, 0, 1);
      }
    else
      {
      $sq = db_rewrite_sql($sq, 'm');
      $q = db_query_range($sq, $tid, 0, 1);
      }
    $ret = db_result($q);
    $ret = $ret > 0;
    return $ret;
    }

  /**
   * Configure the taxonews block identified by $delta
   *
   * @param mixed $delta
   * @return array
   */
  static function blockConfigure($delta)
    {
    $description = t('By default, blocks without matching content are not displayed. This setting allows you to force a static content.');

    $form [self::VAR_EMPTY_MESSAGES] = array
      (
      '#title'         => t('Text to be displayed if block has no matching content'),
      '#type'          => 'textfield',
      '#default_value' => self::getEmptyMessages($delta),
      '#size'          => 60,
      '#max_length'    => 60,
      '#description'   => $description,
      );

    return $form;
    }

  /**
   * Generate block list for hook_block
   *
   * @return array
   * @see taxonews_block
   */
  static function blockList()
    {
    $arBlocks = array() ;
    $arTerms = self::getTerms();
    if (empty($arTerms))
      {
      $modulePath = dirname(drupal_get_filename('module', 'taxonews'));
      drupal_set_message(
        t('WARNING: You will not be able to configure taxonews blocks until you !configure and !define in the taxonews vocabularies. You might want to refer to !install.',
          array(
            '!configure' => l(t('configure taxonews'), self::PATH_SETTINGS),
            '!define'    => l(t('define terms'), 'admin/content/taxonomy'),
            '!install'   => l('INSTALL.txt', "$modulePath/INSTALL.txt")
            )
          ),
        'error');
      unset($modulePath);
      return $arBlocks; // No need to run the following code: there is no block
      }

    $prefix = variable_get(self::VAR_SHOW_NAME, TRUE) ? 'Taxonews/' : '' ;
    $showEmpty = variable_get(self::VAR_SHOW_EMPTY, TRUE);

    $i = 0 ;
    $sq = 'SELECT count(tn.nid) AS cnt '
        . 'FROM {term_node} tn '
        . 'INNER JOIN {node} n ON tn.nid = n.nid '
        . 'WHERE n.status = 1 AND tn.tid = %d ';

    foreach ($arTerms as $term)
      {
      if (!$showEmpty)
        {
        $sq = db_rewrite_sql($sq); // Assuming the "primary" table here is {node}
        $q = db_query($sq, $term->tid);
        $o = db_fetch_object($q);
        if ($o->cnt == 0)
          {
          continue; // Do not display the block, do not increment the counter
          }
        }
      $arBlocks[$i] = array() ;
      $arBlocks[$i]['info'] = "$prefix$term->vocabulary_name/$term->name" ;
      $i++ ;
      }

    return $arBlocks ;
    }

  /**
   * Save the configuration (i.e. the text) of the selected block
   *
   * @param mixed $delta Usually an int
   * @param $edit The edit form fields
   * @return void
   * @see hook_block()
   */
  static function blockSave($delta, $edit)
    {
    $arEmptyMessages = self::getEmptyMessages();
    $arEmptyMessages[$delta] = $edit[self::VAR_EMPTY_MESSAGES];
    variable_set(self::VAR_EMPTY_MESSAGES, $arEmptyMessages);
    }

  /**
   * Generate block contents for the passed-in delta
   *
   * Note: 86400 = 24*60*60 = seconds in one day
   *
   * @param mixed $delta Drupal block delta
   * @return string HTML
   */
  static function blockView($delta)
    {
    $arTerms = self::getTerms() ;

    /**
     * ponderation must only be taken into account if statistics module exists
     * because the module may have been online, allowing ponderation to be set,
     * then removed, causing stats to no longer be updated
     */
    $ponderation = module_exists('statistics')
      ? variable_get(self::VAR_PONDERATED, 0)
      : 0;

    /**
     * There's a small trick for case 0|default: n.created is not a views count, but can
     * be used exactly like one: more recent nodes will have a higher value in
     * this field, so we can sort on it for display just like we sort on actual
     * view counts otherwise. That way we only have one sort rule for all three
     * different cases.
     */
    switch ($ponderation)
      {
      case 1: // by total views
        $sq = 'SELECT n.nid, n.title, td.name, '
            . '  nc.totalcount AS views '
            . 'FROM {node} n '
            . '  INNER JOIN {term_node} tn ON n.nid = tn.nid '
            . '  INNER JOIN {term_data} td ON tn.tid = td.tid '
            . '  LEFT JOIN {node_counter} nc on n.nid = nc.nid ' // Some nodes may never have been counted yet
            . '  /* ignore current time: %d */ ';
        break;

      case 2: // by daily views since creation. Multiply by 86400 to avoid 0-rounding
        $sq = 'SELECT n.nid, n.title, td.name, '
           . '  nc.totalcount*86400/(%d-n.created) AS views '
           . 'FROM {node} n '
           . '  INNER JOIN {term_node} tn ON n.nid = tn.nid '
           . '  INNER JOIN {term_data} td ON tn.tid = td.tid '
           . '  LEFT JOIN {node_counter} nc on n.nid = nc.nid '; // Some nodes may never have been counted yet
         break;

      case 0: // not ponderated
      default: // ignore invalid values
        $sq = 'SELECT n.nid, n.title, td.name, '
            . '  n.created AS views '
            . 'FROM {node} n '
            . '  INNER JOIN {term_node} tn ON n.nid = tn.nid '
            . '  INNER JOIN {term_data} td ON tn.tid = td.tid '
            . '  /* ignore current time: %d */ ';
        break;
      }

    $sq .= 'WHERE td.tid = %d '
       . '  AND (n.created > %d) '
       . '  AND (n.status = 1) '
         . '  ORDER BY n.created DESC, n.changed DESC ';
    $creation = time() - 86400 * variable_get(self::VAR_LIFETIME, self::DEF_LIFETIME);
    $sq = db_rewrite_sql($sq); // Assuming the "primary" table here is {node}
    if ($q = db_query_range ($sq,
      time(), $arTerms[$delta]->tid, $creation,               // current time, tid, lifetime
      0, variable_get(self::VAR_MAX_ROWS, self::DEF_MAX_ROWS) // range
      ))
      {
      $ret = array();
      $items  = array();

      $pos = 0; // Care for missing statistics
      while ($o = db_fetch_object ($q))
        {
        if (empty($o->views)) {
          $o->views = $pos--;
        }
        $items[$o->views] = l($o->title, 'node/' . $o->nid);

        // $o->name is undefined outside the loop, and re-initializing it for every
        // value in the loop seems to actually be the cheapest way of keeping it, although
        // its value is the same for all rows in the result set
        $ret['subject'] = $o->name ;
        }
      krsort($items); // sort descending on views
      }
    if (count($items) == 0)
      {
      $items = NULL;
      $term = taxonomy_get_term($arTerms[$delta]->tid);
      $ret['subject'] = $term->name;
      }

    $ret['content'] = theme('taxonews_block_view', $delta, $items, $arTerms[$delta]->tid);
    return $ret;
    }

  /**
   * Return the list of messages to be issued when an empty taxonews blocks is
   * to be built
   *
   * @param mixed $delta
   * @return array
   */
  static function getEmptyMessages($delta = NULL)
    {
    static $arEmptyMessages;

    if (empty($arEmptyMessages))
      {
      $arEmptyMessages = variable_get(self::VAR_EMPTY_MESSAGES, array());
      }

    $ret = isset($delta)
      ? (array_key_exists($delta, $arEmptyMessages) ? $arEmptyMessages[$delta] : NULL)
      : $arEmptyMessages;

    return $ret;
    }

  /**
   * Query for terms in the vocabularies selected in settings
   *
   * @return array [tid, name]
   */
  static function getTerms()
    {
    static $arTerms = array();

    if (empty($arTerms))
      {
      $arVids = variable_get(self::VAR_VOCABULARY, array());
      $sq = 'SELECT td.tid, td.name '
         . 'FROM {term_data} td '
         . 'WHERE td.vid = %s '
         . 'ORDER BY 2, 1 ';
      foreach ($arVids as $vid)
        {
        $sq = 'SELECT td.tid, td.name, '
            . '  v.name AS vocabulary_name '
            . 'FROM {term_data} td '
            . '  INNER JOIN {vocabulary} v ON td.vid = v.vid '
            . 'WHERE td.vid = %s '
            . 'AND td.vid = v.vid '
            . 'ORDER BY 2, 1 ';
        $sq = db_rewrite_sql($sq, 'v', 'vid');
        $q = db_query($sq, $vid) ;
        while ($o = db_fetch_object($q))
          {
          $arTerms[] = $o;
          }
        }
      }

    return $arTerms;
    }

  } // end of class Taxonews
